//lcd.c
#include "lcd.h"
#include <string.h>
#include <iotms470r1a256.h>

unsigned long data;

void Delay (unsigned long a) { while (--a!=0); }

void LCDInit() {

  LCDInit_low_level();          // Low level init

  LCDSendCommand(DISP_ON);      //LCD on
  LCDSendCommand(CLR_DISP);     //LCD clear
  LCDSendCommand(DD_RAM_ADDR);  //LCD set first row

}


void E_Pulse()
{
  GIODOUTB_bit.GIODOUT0 = 1; //set E to high
  Delay(100);
  GIODOUTB_bit.GIODOUT0 = 0; //set E to low
}

void LCDInit_low_level(void)
{
  HETDIR  |= 0x1;  // light_lcd pin as output

  //first set D4, D5, D6, D7, RS, RW, E to output ports
  GIODIRA_bit.GIODIR4 = 0x1;  // D4
  GIODIRA_bit.GIODIR5 = 0x1;  // D5
  GIODIRA_bit.GIODIR6 = 0x1;  // D6
  GIODIRA_bit.GIODIR7 = 0x1;  // D7
  GIODIRB_bit.GIODIR1 = 0x1;  // RS
  //GIODIRB_bit.GIODIRx = 0x1;  //RW
  GIODIRB_bit.GIODIR0 = 0x1;  // E


  //LCD initialization
  GIODOUTB_bit.GIODOUT1 = 0;  //set RS port to 0
  //GIODOUTB_bit.GIODOUTx = 0;  //set R/W port to 0
  GIODOUTB_bit.GIODOUT0 = 0;  //set E port to 0


  Delay(110000);        //delay ~110ms


  GIODOUTA_bit.GIODOUT4 = 1;  //set D4 port to 1
  GIODOUTA_bit.GIODOUT5 = 1;  //set D5 port to 1
  E_Pulse();                  //high->low to E port (pulse)


  Delay(10000);         //delay ~10ms


  GIODOUTA_bit.GIODOUT4 = 1;  //set D4 port to 1
  GIODOUTA_bit.GIODOUT5 = 1;  //set D5 port to 1
  E_Pulse();                  //high->low to E port (pulse)


  Delay(10000);         //delay ~10ms


  GIODOUTA_bit.GIODOUT4 = 1;  //set D4 port to 1
  GIODOUTA_bit.GIODOUT5 = 1;  //set D5 port to 1
  E_Pulse();                  //high->low to E port (pulse)


  Delay(100000);         //delay ~10ms


  GIODOUTA_bit.GIODOUT5 = 1;  //set D5 port to 1
  GIODOUTA_bit.GIODOUT4 = 0;  //set D4 port to 0
  E_Pulse();                  //high->low to E port (pulse)

}

void LCDSendCommand(unsigned long a)
{
  Delay(4000);                         //delay for LCD char ~2ms
  data = 0x0;

  data = 0xffffff0f | a;                //get high 4 bits
  GIODOUTA &= 0xffffff0f;               //clear D4-D7
  //data = data << 16;

  GIODOUTA = (GIODOUTA | 0x000000f0) & data;  //set D4-D7
  GIODOUTB_bit.GIODOUT1 = 0;                  //set RS port to 0 -> display set to comand mode
  E_Pulse();                                  //pulse to set d4-d7 bits

  // clear data and shift data to get low 4 bytes :-)
  data = 0x0;
  a = a<<4;


  data = 0xffffff0f | a;                //get high 4 bits
  GIODOUTA &= 0xffffff0f;               //clear D4-D7
  //data = data << 16;

  GIODOUTA = (GIODOUTA | 0x000000f0) & data;  //set D4-D7
  GIODOUTB_bit.GIODOUT1 = 0;                  //set RS port to 0 -> display set to comand mode
  E_Pulse();
}

void LCDSendChar(unsigned long a)
{

  Delay(4000);                            //delay for LCD char ~2ms
  data = 0x0;

  data = 0xffffff0f | a;                //get high 4 bits
  GIODOUTA &= 0xffffff0f;               //clear D4-D7
  //data = data << 16;

  GIODOUTA = (GIODOUTA | 0x000000f0) & data;    //set D4-D7
  GIODOUTB_bit.GIODOUT1 = 1;                    //set RS port to 1 -> display set to char mode
  E_Pulse();                                    //pulse to set d4-d7 bits


  // clear data and shift data to get low 4 bytes :-)
  data = 0x0;
  a = a<<4;


  data = 0xffffff0f | a;                //get high 4 bits
  GIODOUTA &= 0xffffff0f;               //clear D4-D7
  //data = data << 16;

  GIODOUTA = (GIODOUTA | 0x000000f0) & data;    //set D4-D7
  GIODOUTB_bit.GIODOUT1 = 1;                    //set RS port to 1 -> display set to char mode
  E_Pulse();                              //pulse to set d4-d7 bits

}

void LCDSendTxt(char* a)
{
  for(int i=0; i<strlen(a); i++)
  {
    LCDSendChar(a[i]);
  }
}

void SmartUp(void)
{
  for(int i=0; i<40; i++) LCDSendCommand(CUR_RIGHT);
}

void SmartDown(void)
{
  for(int i=0; i<40; i++) LCDSendCommand(CUR_LEFT);
}

void LCDLightOn(void) {
  HETDOUT |= 0x1;
}

void LCDLightOff(void) {
  HETDOUT &= ~0x1;
}

